/*
particle.h
Written by Walter O. Krawec

Copyright (c) 2013 Walter O. Krawec

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef PARTICLE_H_
#define PARTICLE_H_

#include <list>
#include "WOK/OpenGL.h"
#include "WOK/timer.h"
#include "Math3D.h"

struct Particle
{
	Particle() : x(0), y(0), bright(1), t(Speck), lifeStage(0) {};
	enum Type
	{
		Speck=0,
		Trail
	};
	float x, y, bright;
	Type t;

	wok::Timer lifeTime;
	int lifeStage;
	void draw()
	{
		if(lifeStage == 0)
			bright = lifeTime.percentComplete();
		else if(lifeStage == 2)
			bright = 1-lifeTime.percentComplete();
		else
			bright = 1;

		if(bright > 1)
			bright = 1;
		else if(bright < 0)
			bright = 0;

		glColor4f(bright, bright, bright, 1);
		glBegin(GL_POINTS);
			glVertex3f(x,y,0);
		glEnd();
		glColor4f(1,1,1,1);

		if(lifeTime.done())
		{
			++lifeStage;
			lifeTime.start();
		}
	}
};

class ParticleManager
{
public:
	void create(float x, float y)
	{
		Particle p;
		p.t = Particle::Trail;
		p.x = x;
		p.y = y;
		p.lifeTime.start(2000);
		p.lifeStage = 1;
		particles.push_back(p);
	}
	void create(wok::gl::Camera& camera, Particle::Type t)
	{
		Particle p;
		p.t = t;
		p.x = rand()%1000;
		p.y = rand()%1000;
		p.x /= 1000.0f;
		p.y /= 1000.0f;

		float range = 50;
		p.x *= 2*range;
		p.y *= 2*range;
		p.x -= range;
		p.y -= range;

		p.x -= camera.xPos();
		p.y -= camera.yPos();

		p.lifeTime.start(1000+rand()%2000);

		particles.push_back(p);
	}
	void draw(wok::gl::Camera& camera)
	{
		if(particles.empty()) return;

		std::list <Particle>::iterator Iter;

		int counter = 0;
		for(Iter = particles.begin(); Iter != particles.end(); ++Iter)
		{
			if(Iter->lifeStage == 3)
			{
				if(Iter->t == Particle::Speck)
					++counter;
				particles.erase(Iter);
				if(particles.empty()) return;
				Iter = particles.begin();
				continue;
			}
			Iter->draw();
		}

		for(int i=0; i<counter; ++i)
			create(camera, Particle::Speck);
	}
private:
	std::list <Particle> particles;
};

class Circle
{
public:
	Circle() : x1(0), y1(0), radius(1){};
		
	virtual void draw(float offsetX, float offsetY, float rot)
	{
		float x = offsetX, y = offsetY;
		int step = (int)(10.0f*radius);
		if(step < 10)
			step = 10;
			
		x += x1;
		y += y1;
		const float d = 6.2832f/step;
		math3D::CVector2D lastPoint,v2;
		lastPoint.Construct(0, radius);
		for(int i=1; i<step; ++i)
		{
			v2.Construct(i*d, radius);
			glBegin(GL_LINES);
			glVertex3f(v2.x+x, v2.y+y, 0);
			glVertex3f(lastPoint.x+x, lastPoint.y+y, 0);
			glEnd();
				
			lastPoint = v2;
		}
			
		v2.Construct(0, radius);
		glBegin(GL_LINES);
		glVertex3f(v2.x+x, v2.y+y, 0);
		glVertex3f(lastPoint.x+x, lastPoint.y+y, 0);
		glEnd();
	}
		
	float x1, y1, radius;
};
class VisualFX
{
public:
	VisualFX() : x(0), y(0), maxR(1), r(1), g(1), b(1) {t.start(1000);};
	VisualFX(float _x, float _y, float _maxR, float _r, float _g, float _b, int msTime) : x(_x), y(_y), maxR(_maxR), r(_r), g(_g), b(_b) {t.start(msTime);};

	bool draw()
	{
		float rad = (maxR - 0.01f)*t.percentComplete() + 0.01f;

		Circle c;
		c.x1 = x;
		c.y1 = y;
		c.radius = rad;

		glColor4f(r,g,b,1);
		c.draw(0,0,0);
		glColor4f(1,1,1,1);

		return t.done();
	}
private:
	wok::Timer t;
	float x, y, maxR;
	float r,g,b;
};

class VisualFXManager
{
public:
	void addFX(VisualFX& fx)
	{
		effects.push_back(fx);
	}

	void draw()
	{
		if(!effects.empty())
		{
			std::list <VisualFX>::iterator Iter;
			for(Iter = effects.begin(); Iter != effects.end(); ++Iter)
			{
				if(Iter->draw())
				{
					effects.erase(Iter);
					break;
				}
			}
		}
	}
private:
	std::list <VisualFX> effects;
};
#endif
